<?
	/**
	 * 	Copyright: Isotope Communications 2008.
	 * 	License: (MIT). See readme.html
	 * 	_________________________________________
	 * 	
	 * 	Drop these methods into your database library (or aggregate them in
	 * 	using your method of choice... )
	 */
	
	
	/**
	 * A wrapper for transactions, registers shutdown_rollback() as
	 * a shutdown function and then creates a new transaction object.
	 *
	 * @return int The id of the transaction that has been created.
	 */
	function start_transaction(){
		
		// First and foremost register the closedown handler
		if ( !$this->_tx_shutdown_registered ){
			//error_log("Registering");
			register_shutdown_function ( array( &$this, "shutdown_rollback" ) );
			//
			$this->_tx_shutdown_registered = 1;
		}
		if ( !$this->_tx_idmap ){
			/**
			 * 	This is not really required anymore.  Speak to Isotope if you want to know
			 * 	what this is about.  It provides a little optimistation for a little extra work on
			 * 	the setup.
			 * 	
			 * 	In short it's a list of PRIMARY_KEYs of your tables.  The transaction library
			 * 	will work this out for itself if it needs to, but if you want to populate this array
			 * 	with a few obvious tables that's fine.
			 */
			$this->_tx_idmap = array();
		}
		
		// Create a new transaction..
		$this->_tx_current =& new transaction( &$this->_tx_idmap );
		$this->_tx_current_id = $this->_tx_current->getID();
		// Register it.
		$this->_tx_reg[ $this->_tx_current_id ] =& $this->_tx_current;
		// Switch to this one as the current one..
		# error_log("INSTANTIATING TRANSACTION [ID:".$this->_tx_current_id."]");
		return ( $this->_tx_current_id );
	}

	/**
	 * Rollsback a transaction, where's _tx_reg coming from?
	 *
	 * @param int $id  The id of the transaction
	 * @return boolean
	 */
	function rollback_transaction( $id ){
		if ( $this->_tx_reg[ $id ]->rollback() ) {
			// If we do rollback.. clean up!
			$this->commit_transaction( $id );
			return ( true );
		}
		return ( false );
	}

	/**
	 * Used to commit a transaction to MySQL, where is _tx_reg set?
	 *
	 * @param int $id The id of the transaction
	 */
	function commit_transaction ( $id ){
		unset ( $this->_tx_reg[ $id ] );
		if ( $this->_tx_current_id ){
			$this->_tx_current_id = false;
			unset ( $this->_tx_current );
		}
	}
	/**
	 * This is registered as a shutdown function in start_transaction
	 * and rollsback any transactions that aren't complete when this
	 * object is thrown away.
	 *
	 */
	function shutdown_rollback( ){
		if ( count($this->_tx_reg) ){
			error_log("[DB:ROLLBACK] Warning :: Automatically rolling back ".count($this->_tx_reg)." transactions" );
			foreach ( $this->_tx_reg as $id => $object ){
				error_log("[DB:ROLLBACK] [Transaction:$id] Warning :: Automatically rolling back ".count($object->_stored_queries)." statements.");
				$this->rollback_transaction( $id );
			}
		}
	}
		
		